# -*- coding: utf-8 -*-


import sys
import warnings


try:
    from boto.s3.connection import S3Connection, OrdinaryCallingFormat
    from boto.s3.key import Key
except ImportError:
    name = 'django_distill.backends.amazon_s3'
    pipm = 'boto'
    sys.stdout.write('{} backend requires {}:\n'.format(name, pipm))
    sys.stdout.write('$ pip install {}\n\n'.format(pipm))
    raise


from django_distill.errors import DistillPublishError
from django_distill.backends import BackendBase
from ssl import CertificateError


class AmazonS3Backend(BackendBase):
    '''
        Publisher for Amazon S3. Implements the BackendBase.
    '''

    REQUIRED_OPTIONS = ('ENGINE', 'PUBLIC_URL', 'ACCESS_KEY_ID',
                        'SECRET_ACCESS_KEY', 'BUCKET')

    def account_username(self):
        return self.options.get('ACCESS_KEY_ID', '')

    def account_container(self):
        return self.options.get('BUCKET', '')

    def authenticate(self, calling_format=None):
        access_key_id = self.account_username()
        secret_access_key = self.options.get('SECRET_ACCESS_KEY', '')
        bucket = self.account_container()
        kwargs = {'calling_format': calling_format} if calling_format else {}
        try:
            self.d['connection'] = S3Connection(access_key_id,
                                                secret_access_key, **kwargs)
            self.d['bucket'] = self.d['connection'].get_bucket(bucket)
        except CertificateError as e:
            # work-around for upstream boto bug for buckets containing dots:
            # https://github.com/boto/boto/issues/2836
            if calling_format:
                raise e
            self.authenticate(calling_format=OrdinaryCallingFormat())

    def list_remote_files(self):
        rtn = set()
        for k in self.d['bucket'].list():
            rtn.add(k.name)
        return rtn

    def delete_remote_file(self, remote_name):
        key = self.d['bucket'].get_key(remote_name)
        return key.delete()

    def compare_file(self, local_name, remote_name):
        key = self.d['bucket'].get_key(remote_name)
        local_hash = self._get_local_file_hash(local_name)
        return local_hash == key.etag[1:-1]

    def upload_file(self, local_name, remote_name):
        k = Key(self.d['bucket'])
        k.key = remote_name
        k.set_contents_from_filename(local_name)
        return True

    def create_remote_dir(self, remote_dir_name):
        # not required for S3 buckets
        return True


backend_class = AmazonS3Backend


# eof



# -*- coding: utf-8 -*-


from django.conf.urls import url


from django_distill.errors import (DistillError, DistillWarning)


urls_to_distill = []


def distill_url(*a, **k):
    distill_func = k.get('distill_func')
    distill_file = k.get('distill_file')
    if distill_file:
        del k['distill_file']
    if distill_func:
        del k['distill_func']
        name = k.get('name')
        if not name:
            raise DistillError('Distill function provided with no name')
        if not callable(distill_func):
            err = 'Distill function not callable: {}'
            raise DistillError(err.format(distill_func))
        urls_to_distill.append((distill_func, distill_file, name, a, k))
    else:
        e = 'URL registered with distill_url but no distill function supplied'
        raise DistillWarning(e)
    return url(*a, **k)


# eof



# -*- coding: utf-8 -*-


import os
import sys
import types
import errno
from shutil import copy2


from future.utils import raise_from


from django.utils import (six, translation)
from django.conf import settings
from django.conf.urls import include as include_urls
from django.http import HttpResponse
from django.template.response import TemplateResponse
from django.test import RequestFactory
from django.urls import reverse
from django.core.management import call_command


from django_distill.errors import (DistillError, DistillWarning)


class DistillRender(object):
    '''
        Renders a complete static site from all urls registered with
        distill_url() and then copies over all static media.
    '''

    def __init__(self, output_dir, urls_to_distill):
        self.output_dir = output_dir
        self.urls_to_distill = urls_to_distill
        # activate the default translation
        translation.activate(settings.LANGUAGE_CODE)

    def render(self):
        for distill_func, file_name, view_name, a, k in self.urls_to_distill:
            for param_set in self.get_uri_values(distill_func):
                if not param_set:
                    param_set = ()
                elif self._is_str(param_set):
                    param_set = param_set,
                uri = self.generate_uri(view_name, param_set)
                render = self.render_view(uri, param_set, a)
                # rewrite URIs ending with a slash to ../index.html
                if file_name is None and uri.endswith('/'):
                    if uri.startswith('/'):
                        uri = uri[1:]
                    yield uri, uri + 'index.html', render
                    continue
                yield uri, file_name, render

    def _is_str(self, s):
        return isinstance(s, six.string_types)

    def get_uri_values(self, func):
        try:
            v = func()
        except Exception as e:
            raise DistillError('Failed to call distill function: {}'.format(e))
        if not v:
            return (None,)
        elif isinstance(v, (list, tuple)):
            return v
        elif isinstance(v, types.GeneratorType):
            return list(v)
        else:
            err = 'Distill function returned an invalid type: {}'
            raise DistillError(err.format(type(v)))

    def generate_uri(self, view_name, param_set):
        if isinstance(param_set, (list, tuple)):
            uri = reverse(view_name, args=param_set)
        elif isinstance(param_set, dict):
            uri = reverse(view_name, kwargs=param_set)
        else:
            err = 'Distill function returned an invalid type: {}'
            raise DistillError(err.format(type(param_set)))
        return uri

    def render_view(self, uri, param_set, args):
        if len(args) < 2:
            raise DistillError('Invalid view arguments')
        view_regex, view_func = args[0], args[1]
        request_factory = RequestFactory()
        request = request_factory.get(uri)
        if isinstance(param_set, dict):
            a, k = (), param_set
        else:
            a, k = param_set, {}
        try:
            response = view_func(request, *a, **k)
        except Exception as err:
            e = 'Failed to render view: {}'.format(err)
            raise_from(DistillError(e), err)
            sys.exit(1)
        if self._is_str(response):
            response = HttpResponse(response)
        elif isinstance(response, TemplateResponse):
            response.render()
        if response.status_code != 200:
            err = 'View returned a non-200 status code: {}'
            raise DistillError(err.format(response.status_code))
        return response

    def copy_static(self, dir_from, dir_to):
        # we need to ignore some static dirs such as 'admin' so this is a
        # little more complex than a straight shutil.copytree()
        if not dir_from.endswith(os.sep):
            dir_from = dir_from + os.sep
        if not dir_to.endswith(os.sep):
            dir_to = dir_to + os.sep
        for root, dirs, files in os.walk(dir_from):
            dirs[:] = filter_dirs(dirs)
            for f in files:
                from_path = os.path.join(root, f)
                base_path = from_path[len(dir_from):]
                to_path = os.path.join(dir_to, base_path)
                to_path_dir = os.path.dirname(to_path)
                if not os.path.isdir(to_path_dir):
                    os.makedirs(to_path_dir)
                copy2(from_path, to_path)
                yield from_path, to_path


def run_collectstatic(stdout):
    stdout('Distill is running collectstatic...')
    call_command('collectstatic')
    stdout('')
    stdout('collectstatic complete, continuing...')


_ignore_dirs = ('admin', 'grappelli')


def filter_dirs(dirs):
    return [d for d in dirs if d not in _ignore_dirs]


def load_urls(stdout):
    stdout('Loading site URLs')
    site_urls = getattr(settings, 'ROOT_URLCONF')
    if site_urls:
        include_urls(site_urls)


def render_to_dir(output_dir, urls_to_distill, stdout):
    mimes = {}
    load_urls(stdout)
    renderer = DistillRender(output_dir, urls_to_distill)
    for page_uri, file_name, http_response in renderer.render():
        if file_name:
            local_uri = file_name
            full_path = os.path.join(output_dir, file_name)
        else:
            local_uri = page_uri
            if page_uri.startswith(os.sep):
                page_uri = page_uri[1:]
            full_path = os.path.join(output_dir, page_uri)
        try:
            content = http_response.content.decode(settings.DEFAULT_CHARSET)
        except Exception as e:
            err = 'Failed to encode {} into {}: {}'
            DistillError(err.format(page_uri, settings.DEFAULT_CHARSET, e))
        mime = http_response.get('Content-Type')
        renamed = ' (renamed from "{}")'.format(page_uri) if file_name else ''
        msg = 'Rendering page: {} -> {} ["{}", {} bytes] {}'
        stdout(msg.format(local_uri, full_path, mime, len(content), renamed))
        try:
            dirname = os.path.dirname(full_path)
            if not os.path.isdir(dirname):
                os.makedirs(dirname)
            with open(full_path, 'w') as f:
                f.write(content)
        except IOError as e:
            if e.errno == errno.EISDIR:
                err = ('Output path: {} is a directory! Try adding a '
                       '"distill_file" arg to your distill_url()')
                raise DistillError(err.format(full_path))
            else:
                raise
        mimes[full_path] = mime.split(';')[0].strip()
    static_url = settings.STATIC_URL
    static_url = static_url[1:] if static_url.startswith('/') else static_url
    static_output_dir = os.path.join(output_dir, static_url)
    for file_from, file_to in renderer.copy_static(settings.STATIC_ROOT,
                                                   static_output_dir):
        stdout('Copying static: {} -> {}'.format(file_from, file_to))
    return True


# eof



# -*- coding: utf-8 -*-


from django_distill.errors import DistillPublishError


def publish_dir(local_dir, backend, stdout):
    stdout('Authenticating')
    backend.authenticate()
    stdout('Getting file indexes')
    remote_files = backend.list_remote_files()
    local_files = backend.list_local_files()
    local_dirs = backend.list_local_dirs()
    to_upload = set()
    to_delete = set()
    local_files_r = set()
    # check local files to upload
    for f in local_files:
        remote_f = backend.remote_path(f)
        local_files_r.add(remote_f)
        if remote_f not in remote_files:
            # local file not present remotely, upload it
            to_upload.add(f)
        else:
            # file is present remotely, check its hash
            if not backend.compare_file(f, remote_f):
                stdout('File stale (hash different): {}'.format(remote_f))
                to_upload.add(f)
            else:
                stdout('File fresh: {}'.format(remote_f))
    # check for remote files to delete
    for f in remote_files:
        if f not in local_files_r:
            to_delete.add(f)
    # upload any new or changed files
    for f in to_upload:
        remote_f = backend.remote_path(f)
        stdout('Publishing: {} -> {}'.format(f, remote_f))
        backend.upload_file(f, backend.remote_path(f))
        url = backend.remote_url(f)
        stdout('Verifying: {}'.format(url))
        if not backend.check_file(f, url):
            err = 'Remote file {} failed hash check'
            raise DistillPublishError(err.format(url))
    # delete any orphan files
    for f in to_delete:
        stdout('Deleting remote: {}'.format(f))
        backend.delete_remote_file(f)


# eof



# -*- coding: utf-8 -*-


import os
from tempfile import mkdtemp
from shutil import rmtree


from django.conf import settings
from django.core.management.base import (BaseCommand, CommandError)


from django_distill.backends import get_backend
from django_distill.distill import urls_to_distill
from django_distill.renderer import (run_collectstatic, render_to_dir)
from django_distill.publisher import publish_dir


try:
    input = raw_input
except NameError:
    pass


class Command(BaseCommand):

    help = 'Distills a site into a temporary local directory then publishes it'

    def add_arguments(self, parser):
        parser.add_argument('publish_target_name', nargs='?', type=str)
        parser.add_argument('--collectstatic', dest='collectstatic',
                            action='store_true')
        parser.add_argument('--quiet', dest='quiet', action='store_true')
        parser.add_argument('--force', dest='force', action='store_true')

    def _quiet(self, *args, **kwargs):
        pass

    def handle(self, *args, **options):
        publish_target_name = options.get('publish_target_name')
        if not publish_target_name:
            publish_target_name = 'default'
        publish_targets = getattr(settings, 'DISTILL_PUBLISH', {})
        publish_target = publish_targets.get(publish_target_name)
        if type(publish_target) != dict:
            e = 'Invalid publish target name: "{}"'.format(publish_target_name)
            e += ', check your settings.DISTILL_PUBLISH values'
            raise CommandError(e)
        publish_engine = publish_target.get('ENGINE')
        if not publish_engine:
            e = 'Publish target {} has no ENGINE'.format(publish_target_name)
            raise CommandError(e)
        collectstatic = options.get('collectstatic')
        quiet = options.get('quiet')
        force = options.get('force')
        if quiet:
            stdout = self._quiet
        else:
            stdout = self.stdout.write
        static_dir = settings.STATIC_ROOT
        static_url = settings.STATIC_URL
        try:
            output_dir = mkdtemp()
            if not output_dir.endswith(os.sep):
                output_dir += os.sep
            backend_class = get_backend(publish_engine)
            backend = backend_class(output_dir, publish_target)
            username = backend.account_username()
            container = backend.account_container()
            stdout('')
            stdout('You have requested to distill and publish this site')
            stdout('to the following target:')
            stdout('')
            stdout('    Settings name: {}'.format(publish_target_name))
            stdout('    Engine:        {}'.format(publish_engine))
            stdout('    Username:      {}'.format(username))
            stdout('    Container:     {}'.format(container))
            stdout('')
            if collectstatic:
                run_collectstatic(stdout)
            if not os.path.isdir(settings.STATIC_ROOT):
                e = 'Static source directory does not exist, run collectstatic'
                raise CommandError(e)
            ans = input('Type \'yes\' to continue, or \'no\' to cancel: ')
            if ans == 'yes':
                pass
            else:
                raise CommandError('Publishing site cancelled.')
            self.stdout.write('')
            static_output_dir = os.path.join(output_dir, static_url[1:])
            msg = 'Generating static site into directory: {}'
            stdout(msg.format(output_dir))
            render_to_dir(output_dir, urls_to_distill, stdout)
            stdout('')
            stdout('Publishing site')
            backend.index_local_files()
            publish_dir(output_dir, backend, stdout)
        finally:
            if os.path.exists(output_dir):
                stdout('Deleting temporary directory.')
                rmtree(output_dir)
        stdout('')
        stdout('Site generation and publishing complete.')


# eof



# -*- coding: utf-8 -*-


import sys
import warnings


try:
    import pyrax
except ImportError:
    name = 'django_distill.backends.rackspace_files'
    pipm = 'pyrax'
    sys.stdout.write('{} backend requires {}:\n'.format(name, pipm))
    sys.stdout.write('$ pip install {}\n\n'.format(pipm))
    raise


from django_distill.errors import DistillPublishError
from django_distill.backends import BackendBase


class RackspaceCloudFilesBackend(BackendBase):
    '''
        Publisher for Rackspace Cloud Files. Implements the BackendBase.
    '''

    REQUIRED_OPTIONS = ('ENGINE', 'PUBLIC_URL', 'USERNAME', 'API_KEY',
                        'REGION', 'CONTAINER')

    def account_username(self):
        return self.options.get('USERNAME', '')

    def account_container(self):
        return self.options.get('CONTAINER', '')

    def authenticate(self):
        username = self.account_username()
        api_key = self.options.get('API_KEY', '')
        region = self.options.get('REGION', '')
        container = self.account_container()
        pyrax.set_setting('identity_type', 'rackspace')
        with warnings.catch_warnings():
            warnings.simplefilter('ignore')
            pyrax.set_credentials(username, api_key, region=region)
        self.d['connection'] = pyrax.cloudfiles
        if not self.d['connection']:
            e = 'Failed to connect to Rackspace Cloud Files, check credentials'
            raise DistillPublishError(e)
        self.d['container'] = self.d['connection'].get_container(container)

    def list_remote_files(self):
        rtn = set()
        marker, limit = 100, ''
        while True:
            objects = self.d['container'].get_objects(limit=limit,
                                                      marker=marker)
            if not objects:
                break
            for obj in objects:
                rtn.add(obj.name)
            marker = objects[-1].name
        return rtn

    def delete_remote_file(self, remote_name):
        return self.d['container'].delete_object(remote_name)

    def compare_file(self, local_name, remote_name):
        if not self._file_exists(local_name):
            raise DistillPublishError('File does not exist: {}'.format(
                local_name))
        local_hash = self._get_local_file_hash(local_name)
        o = self.d['container'].get_object(remote_name)
        return o.etag == local_hash

    def upload_file(self, local_name, remote_name):
        if not self._file_exists(local_name):
            raise DistillPublishError('File does not exist: {}'.format(
                local_name))
        local_hash = self._get_local_file_hash(local_name)
        remote_obj = self.d['container'].upload_file(local_name, remote_name,
                                                     etag=local_hash)
        return local_hash == remote_obj.etag

    def create_remote_dir(self, remote_dir_name):
        # Rackspace Files containers have no directories
        pass


backend_class = RackspaceCloudFilesBackend


# eof



# -*- coding: utf-8 -*-

import os
import sys
from setuptools import (setup, find_packages)


version = 1.1


setup(
    name='django-distill',
    version=str(version),
    url='https://github.com/mgrp/django-distill',
    download_url='https://github.com/mgrp/django-distill/tarball/0.6',
    author='the m group, https://m.pr/',
    author_email='hi@m.pr',
    description=('Static site renderer and publisher for Django.'),
    license='MIT',
    include_package_data=True,
    install_requires=('django', 'requests', 'future'),
    packages=find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords=('django', 'distill', 'static', 's3', 'rackspace',
              'google cloud storage'),
)

# eof



# -*- coding: utf-8 -*-


import os
from binascii import hexlify
from tempfile import NamedTemporaryFile


from django.conf import settings
from django.core.management.base import (BaseCommand, CommandError)


from django_distill.backends import get_backend


try:
    input = raw_input
except NameError:
    pass


class Command(BaseCommand):

    help = 'Tests a distill publishing target'

    def add_arguments(self, parser):
        parser.add_argument('publish_target_name', nargs='?', type=str)

    def handle(self, *args, **options):
        publish_target_name = options.get('publish_target_name')
        if not publish_target_name:
            publish_target_name = 'default'
        publish_targets = getattr(settings, 'DISTILL_PUBLISH', {})
        publish_target = publish_targets.get(publish_target_name)
        if type(publish_target) != dict:
            e = 'Invalid publish target name: "{}"'.format(publish_target_name)
            e += ', check your settings.DISTILL_PUBLISH values'
            raise CommandError(e)
        publish_engine = publish_target.get('ENGINE')
        if not publish_engine:
            e = 'Publish target {} has no ENGINE'.format(publish_target_name)
            raise CommandError(e)
        self.stdout.write('')
        self.stdout.write('You have requested to test a publishing target:')
        self.stdout.write('')
        self.stdout.write('    Name:   {}'.format(publish_target_name))
        self.stdout.write('    Engine: {}'.format(publish_engine))
        self.stdout.write('')
        ans = input('Type \'yes\' to continue, or \'no\' to cancel: ')
        if ans == 'yes':
            self.stdout.write('')
            self.stdout.write('Testing publishing target...')
        else:
            raise CommandError('Testing publishing target cancelled.')
        self.stdout.write('')
        self.stdout.write('Connecting to backend engine')
        backend_class = get_backend(publish_engine)
        random_file = NamedTemporaryFile(delete=False)
        random_str = hexlify(os.urandom(16))
        random_name = '/' + random_str.decode('utf-8')
        random_file.write(random_str)
        random_file.close()
        backend = backend_class(os.path.dirname(random_file.name),
                                publish_target)
        self.stdout.write('Authenticating')
        backend.authenticate()
        remote_file_name = os.path.basename(random_file.name)
        self.stdout.write('Uploading test file: {}'.format(random_file.name))
        backend.upload_file(random_file.name, remote_file_name)
        url = backend.remote_url(random_file.name)
        self.stdout.write('Verifying remote test file: {}'.format(url))
        if backend.check_file(random_file.name, url):
            self.stdout.write('File uploaded correctly, file hash is correct')
        else:
            msg = 'File error, remote file hash differs from local hash'
            self.stderr.write(msg)
        self.stdout.write('Deleting remote test file')
        backend.delete_remote_file(remote_file_name)
        if os.path.exists(random_file.name):
            self.stdout.write('Deleting local test file')
            os.unlink(random_file.name)
        self.stdout.write('')
        self.stdout.write('Backend testing complete.')


# eof
